/*
** DO NOT EDIT THIS FILE !!!
** It was generated automatically by:
**
**   User: jan
**   Host: caesar
**   Date: 2015-03-18 16:20:01
**   Prog: ./mktuple
**
** Purpose:
**   Emulating variadic templates by generating appropriate
**   specializations and overloads.
*/
#ifndef DOXYGEN
template
<
    typename T0 = OFtuple_nil,
    typename T1 = OFtuple_nil,
    typename T2 = OFtuple_nil,
    typename T3 = OFtuple_nil,
    typename T4 = OFtuple_nil,
    typename T5 = OFtuple_nil,
    typename T6 = OFtuple_nil,
    typename T7 = OFtuple_nil,
    typename T8 = OFtuple_nil,
    typename T9 = OFtuple_nil,
    typename T10 = OFtuple_nil,
    typename T11 = OFtuple_nil,
    typename T12 = OFtuple_nil,
    typename T13 = OFtuple_nil,
    typename T14 = OFtuple_nil,
    typename T15 = OFtuple_nil,
    typename T16 = OFtuple_nil,
    typename T17 = OFtuple_nil,
    typename T18 = OFtuple_nil,
    typename T19 = OFtuple_nil,
    typename T20 = OFtuple_nil,
    typename T21 = OFtuple_nil,
    typename T22 = OFtuple_nil,
    typename T23 = OFtuple_nil,
    typename T24 = OFtuple_nil,
    typename T25 = OFtuple_nil,
    typename T26 = OFtuple_nil,
    typename T27 = OFtuple_nil,
    typename T28 = OFtuple_nil,
    typename T29 = OFtuple_nil,
    typename T30 = OFtuple_nil,
    typename T31 = OFtuple_nil,
    typename T32 = OFtuple_nil,
    typename T33 = OFtuple_nil,
    typename T34 = OFtuple_nil,
    typename T35 = OFtuple_nil,
    typename T36 = OFtuple_nil,
    typename T37 = OFtuple_nil,
    typename T38 = OFtuple_nil,
    typename T39 = OFtuple_nil,
    typename T40 = OFtuple_nil,
    typename T41 = OFtuple_nil,
    typename T42 = OFtuple_nil,
    typename T43 = OFtuple_nil,
    typename T44 = OFtuple_nil,
    typename T45 = OFtuple_nil,
    typename T46 = OFtuple_nil,
    typename T47 = OFtuple_nil,
    typename T48 = OFtuple_nil,
    typename T49 = OFtuple_nil
>
struct OFtuple;
#endif // DOXYGEN
